/*----------------------------------------------------------------------- 
 * 
 * File Name: FindChirpClusterEvents.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: FindChirpClusterEvents.c,v 1.7 2005/04/22 16:26:13 babak Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="FindChirpClusterEventsCV">
Author: Brown D. A., Woods D.
$Id: FindChirpClusterEvents.c,v 1.7 2005/04/22 16:26:13 babak Exp $
</lalVerbatim>

<lalLaTeX>
\subsection{Module \texttt{FindChirpClusterEvents.c}}
\label{ss:FindChirpClusterEvents.c}

</lalLaTeX>
#endif

#include <math.h>
#include <lal/LALStdio.h>
#include <lal/LALStdlib.h>
#include <lal/LALConstants.h>
#include <lal/Date.h>
#include <lal/AVFactories.h>
#include <lal/FindChirp.h>

#define rint(x) (floor((x)+0.5))

NRCSID (FINDCHIRPCLUSTEREVENTSC, "$Id: FindChirpClusterEvents.c,v 1.7 2005/04/22 16:26:13 babak Exp $"); 

/* <lalVerbatim file="FindChirpClusterEventsCP"> */
void
LALFindChirpClusterEvents (
    LALStatus                  *status,
    SnglInspiralTable         **eventList,
    FindChirpFilterInput       *input,
    FindChirpFilterParams      *params,
    COMPLEX8                   *q,
    UINT4                       kmax,
    UINT4                       numPoints,
    UINT4                       ignoreIndex,
    REAL4                       norm,
    REAL4                       modqsqThresh,
    REAL4                       chisqThreshFac,
    UINT4                       numChisqBins,
    CHAR                        searchName[LIGOMETA_SEARCH_MAX] 
		)
/* </lalVerbatim> */
{
  UINT4                 eventStartIdx = 0;
  UINT4  		deltaEventIndex = 0;
  UINT4                 j;
  SnglInspiralTable    *thisEvent = NULL;

  INITSTATUS( status, "LALFindChirpClusterEvents", FINDCHIRPCLUSTEREVENTSC );
  ATTATCHSTATUSPTR( status );


  /*
   *
   * check that the arguments are reasonable
   *
   */


  /* make sure the output handle exists, but points to a null pointer */
  ASSERT( eventList, status, FINDCHIRPH_ENULL, FINDCHIRPH_MSGENULL );
  ASSERT( !*eventList, status, FINDCHIRPH_ENNUL, FINDCHIRPH_MSGENNUL );

  /*
   *
   * Apply one of the clustering algorithms
   *
   */

   /* set deltaEventIndex depending on clustering method used */
   if ( params->clusterMethod == tmplt )
   {
     deltaEventIndex = (UINT4) rint( (input->fcTmplt->tmplt.tC / params->deltaT) + 1.0 );
   }
   else if ( params->clusterMethod == window )
   {
     deltaEventIndex = (UINT4) rint( (params->clusterWindow / params->deltaT) + 1.0 );
   }


  /* look for an events in the filter output */
  for ( j = ignoreIndex; j < numPoints - ignoreIndex; ++j )
  {
    REAL4 modqsq = q[j].re * q[j].re + q[j].im * q[j].im;

    /* if snrsq exceeds threshold at any point */
    if ( modqsq > modqsqThresh )
    {

      /* if we have don't have a chisq or the chisq drops below the       */
      /* modified chisq threshold, start processing events                */
      if ( ! input->segment->chisqBinVec->length ||
          params->chisqVec->data[j] < 
          (params->chisqThresh * ( 1.0 + modqsq * chisqThreshFac )) )
      {


        /*
         *
         * find the local maximum of the event
         *
         */


        if ( ! *eventList )
        {
          /* store the start of the crossing */
          eventStartIdx = j;

          /* if this is the first event, start the list */
          thisEvent = *eventList = (SnglInspiralTable *) 
            LALCalloc( 1, sizeof(SnglInspiralTable) );
          if ( ! thisEvent )
          {
            ABORT( status, FINDCHIRPH_EALOC, FINDCHIRPH_MSGEALOC );
          }

          /* record the data that we need for the clustering algorithm */
          thisEvent->end_time.gpsSeconds = j;
          thisEvent->snr = modqsq;
        }
        else if (  ! params->clusterMethod == noClustering  &&
            j <= thisEvent->end_time.gpsSeconds + deltaEventIndex &&
            modqsq > thisEvent->snr )
        {
          /* if this is the same event, update the maximum */
          thisEvent->end_time.gpsSeconds = j;
          thisEvent->snr = modqsq;
        }
        else if ( j > thisEvent->end_time.gpsSeconds + deltaEventIndex ||
              params->clusterMethod == noClustering  )
        {
          /* clean up this event */
          SnglInspiralTable *lastEvent;
          LALFindChirpStoreEvent(status->statusPtr, input, params,
              thisEvent, q, kmax, norm, eventStartIdx, numChisqBins, 
              searchName );
          CHECKSTATUSPTR( status );

          /* store the start of the crossing */
          eventStartIdx = j;

          /* allocate memory for the newEvent */
          lastEvent = thisEvent;

          lastEvent->next = thisEvent = (SnglInspiralTable *) 
            LALCalloc( 1, sizeof(SnglInspiralTable) );
          if ( ! lastEvent->next )
          {
            ABORT( status, FINDCHIRPH_EALOC, FINDCHIRPH_MSGEALOC );
          }

          /* stick minimal data into the event */
          thisEvent->end_time.gpsSeconds = j;
          thisEvent->snr = modqsq;
        }
      }
    }
  }

  /* 
   *
   * clean up last event
   *
   */

  if ( thisEvent )
  {
    LALFindChirpStoreEvent(status->statusPtr, input, params,
         thisEvent, q, kmax, norm, eventStartIdx, numChisqBins, 
         searchName );
    CHECKSTATUSPTR( status );
  }

  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
